@@ -66,6 +66,11 @@ class CustomParser(argparse.ArgumentParser):
         self.finished_init = True
 
     def parse_entry_point_value(self):
+        """
+        Returns the entry_point value pased without calling parse_args(),
+        this is required to find env params to show, if we call parse_args()
+        the CLI stops there and shows available params
+        """
         index = None
 
         try:
@@ -84,7 +89,7 @@ class CustomParser(argparse.ArgumentParser):
     def add_argument(self, *args, **kwargs):
         if not self.finished_static_api:
             if not self.in_context and self.finished_init:
-                raise RuntimeError('Cannot aadd arguments until the static '
+                raise RuntimeError('Cannot add arguments until the static '
                                    'API has been declared')
             else:
                 self.static_args.extend([process_arg(arg) for arg in args])
@@ -98,37 +103,83 @@ class CustomParser(argparse.ArgumentParser):
         self.in_context = False
         self.finished_static_api = True
 
+    def process_factory_dotted_path(self, dotted_path):
+        """Parse a factory entry point, returns initialized dag and parsed args
+        """
+        entry = load_dotted_path(dotted_path, raise_=True)
 
-def _parse_doc(doc):
+        # add args using the function's signature
+        required, _ = _add_args_from_callable(self, entry)
+
+        # if entry point was decorated with @with_env, add arguments
+        # to replace declared variables in env.yaml
+        if hasattr(entry, '_env_dict'):
+            _add_args_from_env_dict(self, entry._env_dict)
+
+        args = self.parse_args()
+
+        if hasattr(args, 'log'):
+            if args.log is not None:
+                logging.basicConfig(level=args.log.upper())
+
+        # extract required (by using function signature) params from the cli
+        # args
+        kwargs = {key: getattr(args, key) for key in required}
+
+        # env and function defaults replaced
+        replaced = _args_to_replace_in_env(args, self.static_args)
+
+        # TODO: add a way of test this by the parameters it will use to
+        # call the function, have an aux function to get those then another
+        # to execute, test using the first one
+        dag = entry(**{**kwargs, **replaced})
+
+        return dag, args
+
+
+def _first_non_empty_line(doc):
+    for line in doc.split('\n'):
+        if line:
+            return line.strip()
+
+
+def _parse_doc(callable_):
     """
     Convert numpydoc docstring to a list of dictionaries
     """
+    doc = callable_.__doc__
+
     # no docstring
     if doc is None:
         return {'params': {}, 'summary': None}
 
-    # try to import numpydoc
-    docscrape = importlib.import_module('numpydoc.docscrape')
+    # try to import numpydoc, if can't find it, just returnt the first line
+    try:
+        docscrape = importlib.import_module('numpydoc.docscrape')
+    except ModuleNotFoundError:
+        return {'params': {}, 'summary': _first_non_empty_line(doc)}
 
-    if not docscrape:
-        return {'params': {}, 'summary': None}
+    doc_parsed = docscrape.NumpyDocString(doc)
 
-    doc = docscrape.NumpyDocString(doc)
     parameters = {
         p.name: {
             'desc': ' '.join(p.desc),
             'type': p.type
         }
-        for p in doc['Parameters']
+        for p in doc_parsed['Parameters']
     }
-    summary = doc['Summary']
+
+    # docscrape returns one element per line
+    summary = 'Docstring: {}'.format('\n'.join(doc_parsed['Summary']))
+
     return {'params': parameters, 'summary': summary}
 
 
 def _args_to_replace_in_env(args, static_args):
     """
-    Returns a dictionary with all extra parameters passed, all these must
-    be parameters to replace env values
+    Returns a dictionary with all extra cli parameters passed, all these must
+    be parameters that part of the env or params (with no defaults) if
+    entry point is a factory function
     """
     return {
         name: getattr(args, name)
@@ -142,9 +193,17 @@ def _add_args_from_env_dict(parser, env_dict):
     Add one parameter to the args parser by taking a look at all values
     defined in an env dict object
     """
+    # flatten keys from the env dictionary. e.g. from {'a': {'b': 1}} is
+    # converted to {'a--b': 1}. This allows us to add cli args such as --a--b
+    # to modify any key in the env. Note that we use double hyphens to have
+    # an unambious section separator. Environments are commonly loaded from
+    # YAML files. Keys in such files might contain hyphens/underscores, we
+    # allow users to have those characters but double hyphens/underscores are
+    # not permitted as they'd conflict with the CLI generation logic
     flat_env_dict = _flatten_dict(env_dict._data)
+
     for arg, val in flat_env_dict.items():
-        parser.add_argument('--env__' + arg, help='Default: {}'.format(val))
+        parser.add_argument('--env--' + arg, help='Default: {}'.format(val))
 
 
 def _parse_signature_from_callable(callable_):
@@ -177,10 +236,16 @@ def add_argument_kwargs(params, arg):
     """
     valid_hints = [int, float, str, bool]
 
-    if params[arg].annotation in valid_hints:
-        kwargs = {'type': params[arg].annotation}
+    fn_default = params[arg].default
+    fn_annotation = params[arg].annotation
+
+    # special case, bool with default value becomes a flag
+    if fn_annotation is bool and fn_default is not inspect._empty:
+        kwargs = {'action': 'store_true' if not fn_default else 'store_false'}
+    elif fn_annotation in valid_hints:
+        kwargs = {'type': fn_annotation, 'default': fn_default}
     else:
-        kwargs = {}
+        kwargs = {'default': fn_default}
 
     return kwargs
 
@@ -195,7 +260,7 @@ def _add_args_from_callable(parser, callable_):
 
     Returns parsed args: required (list) and defaults (dict)
     """
-    doc = _parse_doc(callable_.__doc__)
+    doc = _parse_doc(callable_)
     required, defaults, params = _parse_signature_from_callable(callable_)
 
     for arg, default in defaults.items():
@@ -209,19 +274,21 @@ def _add_args_from_callable(parser, callable_):
                             **add_argument_kwargs(params, arg))
 
     if doc['summary']:
-        # summary should return the first line only, but we do this just in
-        # case the numpydoc implementation changes
-        summary = '\n'.join(doc['summary'])
         desc = parser.description
-        parser.description = '{}. Docstring: {}'.format(desc, summary)
+        parser.description = '{}. {}'.format(desc, doc['summary'])
 
     return required, defaults
 
 
-def _process_file_or_entry_point(parser, entry_point, static_args):
+def _process_file_or_entry_point(parser):
     """
     Process a file entry point file or directory), returns the initialized dag
     and parsed args
+
+    Parameters
+    ----------
+    parser : CustomParser
+        CLI arg parser
     """
     if Path('env.yaml').exists():
         env_dict = EnvDict('env.yaml')
@@ -233,52 +300,21 @@ def _process_file_or_entry_point(parser, entry_point, static_args):
         if args.log is not None:
             logging.basicConfig(level=args.log.upper())
 
-    if Path(entry_point).is_dir():
-        dag = DAGSpec.from_directory(entry_point).to_dag()
+    if Path(args.entry_point).is_dir():
+        dag = DAGSpec.from_directory(args.entry_point).to_dag()
     else:
-        with open(entry_point) as f:
+        with open(args.entry_point) as f:
             dag_dict = yaml.load(f, Loader=yaml.SafeLoader)
 
+        # load env.yaml if there is one
         if Path('env.yaml').exists():
             env = EnvDict('env.yaml')
-            replaced = _args_to_replace_in_env(args, static_args)
+            # and replace keys depending on passed cli args
+            replaced = _args_to_replace_in_env(args, parser.static_args)
             env = env._replace_flatten_keys(replaced)
             dag = DAGSpec(dag_dict, env=env).to_dag()
         else:
-            dag = DAGSpec(dag_dict).to_dag()
-
-    return dag, args
-
-
-def _process_factory_dotted_path(parser, dotted_path, static_args):
-    """Parse a factory entry point, returns initialized dag and parsed args
-
-    """
-    entry = load_dotted_path(dotted_path, raise_=True)
-
-    required, _ = _add_args_from_callable(parser, entry)
-
-    # if entry point was decorated with @with_env, add arguments
-    # to replace declared variables in env.yaml
-    if hasattr(entry, '_env_dict'):
-        _add_args_from_env_dict(parser, entry._env_dict)
-
-    args = parser.parse_args()
-
-    if hasattr(args, 'log'):
-        if args.log is not None:
-            logging.basicConfig(level=args.log.upper())
-
-    # extract required (by using function signature) params from the cli args
-    kwargs = {key: getattr(args, key) for key in required}
-
-    # env and function defaults replaced
-    replaced = _args_to_replace_in_env(args, static_args)
-
-    # TODO: add a way of test this by the parameters it will use to
-    # call the function, have an aux function to get those then another
-    # to execute, test using the first one
-    dag = entry(**{**kwargs, **replaced})
+            dag = DAGSpec(dag_dict, env=None).to_dag()
 
     return dag, args
 
@@ -310,8 +346,7 @@ def _process_entry_point(parser, entry_point, static_args):
                       'exist'.format(entry_point, path.suffix))
 
     # even if the entry file is not a file nor a valid module, show the help
-    # menu, but show a warning because this will prevent pipeline parameters
-    # from showing up
+    # menu, but show a warning
     if (help_cmd and not entry_file_exists and not entry_obj):
         warnings.warn('Failed to load entry point "{}". It is not a file '
                       'nor a valid dotted path'.format(entry_point))
@@ -320,12 +355,10 @@ def _process_entry_point(parser, entry_point, static_args):
 
     # first check if the entry point is an existing file
     elif path.exists():
-        dag, args = _process_file_or_entry_point(parser, entry_point,
-                                                 static_args)
+        dag, args = _process_file_or_entry_point(parser)
     # assume it's a dotted path to a factory
     else:
-        dag, args = _process_factory_dotted_path(parser, entry_point,
-                                                 static_args)
+        dag, args = parser.process_factory_dotted_path(entry_point)
 
     return dag, args
 
@@ -363,7 +396,7 @@ def load_entry_point(entry_point):
         spec = DAGSpec.from_directory(entry_point)
         path = Path(entry_point)
     elif type_ == 'file':
-        spec = DAGSpec.from_file(entry_point)
+        spec = DAGSpec(entry_point)
         path = Path(entry_point).parent
     elif type_ == 'dotted-path':
         raise ValueError('dotted paths are currently unsupported')
@@ -387,13 +420,13 @@ def _custom_command(parser):
 
 def _flatten_dict(d, prefix=''):
     """
-    Convert a nested dict: {'a': {'b': 1}} -> {'a__b': 1}
+    Convert a nested dict: {'a': {'b': 1}} -> {'a--b': 1}
     """
     out = {}
 
     for k, v in d.items():
         if isinstance(v, Mapping):
-            out = {**out, **_flatten_dict(v, prefix=prefix + k + '__')}
+            out = {**out, **_flatten_dict(v, prefix=prefix + k + '--')}
         else:
             out[prefix + k] = v
 
