@@ -10,7 +10,6 @@ meta:
 All other sections should represent valid DAG properties.
 """
 import os
-import sys
 import yaml
 import logging
 from pathlib import Path
@@ -22,11 +21,12 @@ import pprint
 from ploomber import products
 from ploomber import DAG, tasks, SourceLoader
 from ploomber.util.util import (load_dotted_path, find_file_recursively,
-                                call_with_dictionary)
+                                call_with_dictionary, add_to_sys_path)
 from ploomber.spec.TaskSpec import TaskSpec, suffix2taskclass
 from ploomber.util import validate
 from ploomber.dag.DAGConfiguration import DAGConfiguration
 from ploomber.exceptions import DAGSpecInitializationError
+from ploomber.env.EnvDict import EnvDict
 from ploomber.env.expand import expand_raw_dictionary
 from ploomber.tasks import NotebookRunner
 
@@ -43,6 +43,17 @@ class DAGSpec(MutableMapping):
     "location" key with the factory to call, the other explicitly describes
     the DAG structure as a dictionary.
 
+    Parameters
+    ----------
+    data : dict
+        Specification to construct the DAG
+
+    env : dict, pathlib.path or str
+        Dictionary or path/str to a YAML file with the environment to use,
+        tags in any of the keys (i.e. {{some_tag}}) in "data" will be replaced
+        by the corresponding values in "env". A regular :py:mod:`ploomber.Env`
+        object is created, see documentation for details
+
     Notes
     -----
     Relative paths are heavily used for pointing to source files and products,
@@ -60,23 +71,30 @@ class DAGSpec(MutableMapping):
     relative to the source file or pipeline.yaml? To make this consistent,
     all relative paths (whether in a pipeline.yaml or in a source file) are
     relative to the pipeline.yaml folder but this can be changed with the
-    product_relative_to_source option. Furthermore, when usign the custom
+    product_relative_to_source option. Furthermore, when using the custom
     contents manager, we convert relative paths to absolute in special cases
     to make sure they work. This is why we need to keep the pipeline.yaml
     location when initializing a DAG from a spec.
     """
     def __init__(self, data, env=None):
-        # only set when initialized from DAGSpec.from_file
-        self._parent_path = None
+        if isinstance(data, (str, Path)):
+            self._parent_path = str(Path(data).parent)
+
+            with open(str(data)) as f:
+                data = yaml.load(f, Loader=yaml.SafeLoader)
+        else:
+            self._parent_path = None
 
         if isinstance(data, list):
             data = {'tasks': data}
 
         logger.debug('DAGSpec enviroment:\n%s', pp.pformat(env))
 
-        # expand if there's an env.yaml file
         if env is not None:
-            data = expand_raw_dictionary(data, env)
+            self.env = EnvDict(env)
+            data = expand_raw_dictionary(data, self.env)
+        else:
+            self.env = None
 
         logger.debug('Expanded DAGSpec:\n%s', pp.pformat(data))
 
@@ -89,9 +107,12 @@ class DAGSpec(MutableMapping):
             ]
             self._validate_meta()
 
-            self.data['tasks'] = [
-                TaskSpec(t, self.data['meta']) for t in self.data['tasks']
-            ]
+            # make sure the folder where the pipeline is located is in sys.path
+            # otherwise dynamic imports needed by TaskSpec will fail
+            with add_to_sys_path(self._parent_path, chdir=False):
+                self.data['tasks'] = [
+                    TaskSpec(t, self.data['meta']) for t in self.data['tasks']
+                ]
 
     def _validate_top_level_keys(self, spec):
         load_from_factory = False
@@ -133,25 +154,17 @@ class DAGSpec(MutableMapping):
     def to_dag(self):
         """Converts the DAG spec to a DAG object
         """
-        cwd_old = os.getcwd()
-
         # when initializing DAGs from pipeline.yaml files, we have to ensure
-        # that the parent folder is in sys.path for imports to work, this
-        # happens most of the time but for some (unknown) reason, it doesn't
+        # that the folder where pipeline.yaml is located is in sys.path for
+        # imports to work (for dag clients), this happens most of the time but
+        # for some (unknown) reason, it doesn't
         # happen when initializing PloomberContentsManager.
         # pipeline.yaml paths are written relative to that file, for source
         # scripts to be located we temporarily change the current working
         # directory
-        if self._parent_path is not None:
-            sys.path.insert(0, os.path.abspath(self._parent_path))
-            os.chdir(self._parent_path)
-
-        try:
+        with add_to_sys_path(self._parent_path, chdir=True):
             dag = self._to_dag()
-        finally:
-            if self._parent_path is not None:
-                sys.path.remove(os.path.abspath(self._parent_path))
-                os.chdir(cwd_old)
+
         return dag
 
     def _to_dag(self):
@@ -177,7 +190,7 @@ class DAGSpec(MutableMapping):
         return dag
 
     @classmethod
-    def auto_load(cls, to_dag=True, starting_dir=None):
+    def auto_load(cls, to_dag=True, starting_dir=None, env=None):
         """
         Looks for a pipeline.yaml, generates a DAGSpec and returns a DAG.
         Currently, this is only used by the PloomberContentsManager, this is
@@ -202,7 +215,7 @@ class DAGSpec(MutableMapping):
                 return None, None
 
         try:
-            spec = cls.from_file(path)
+            spec = cls(path, env=env)
 
             if to_dag:
                 return spec, spec.to_dag(), Path(path).parent
@@ -222,6 +235,11 @@ class DAGSpec(MutableMapping):
         interpret them as task sources, file names are assigned as task names
         (without extension). The spec is generated with the default values in
         the "meta" section.
+
+        Notes
+        -----
+        ``env`` is not supported because the spec is generated from files
+        in ``path_to_dir``, hence, there is no way to embed tags
         """
         extensions = [
             name for name, class_ in suffix2taskclass.items()
@@ -238,18 +256,6 @@ class DAGSpec(MutableMapping):
 
         return cls({'tasks': tasks})
 
-    @classmethod
-    def from_file(cls, path, env=None):
-        """
-        Initialize dag spec with yaml file
-        """
-        with open(str(path)) as f:
-            dag_dict = yaml.load(f, Loader=yaml.SafeLoader)
-
-        spec = cls(dag_dict, env=env)
-        spec._parent_path = str(Path(path).parent)
-        return spec
-
 
 class Meta:
     """Schema for meta section in pipeline.yaml
@@ -358,6 +364,8 @@ def init_clients(dag, clients):
 
 
 def normalize_task(task):
+    # NOTE: we haven't documented that we support tasks as just strings,
+    # should we keep this or deprecate it?
     if isinstance(task, str):
         return {'source': task}
     else:
