@@ -42,6 +42,11 @@ When a Task is marked as Errored, on_failure hook is executed and downstream
 dependencies are marked as Aborted.
 
 
+Beware that sometimes executors call Task._build in a different process, hence
+updates won't be visible in Task objects that exist in the main process.
+For that reason, executors have to report back status to the original Task
+object using Task.exec_status, see executors documentation for details
+
 TODO: describe BrokenProcesssPool status
 """
 import abc
@@ -284,13 +289,15 @@ class Task(abc.ABC):
             try:
                 self.on_finish(**kwargs)
             except Exception as e:
+                # FIXME: There's duplicated logic here, _build also catches
+                # this errors
                 msg = ('Exception when running on_finish '
                        'for task "{}": {}'.format(self.name, e))
                 self._logger.exception(msg)
                 self.exec_status = TaskStatus.Errored
                 raise type(e)(msg) from e
 
-        self.product._save_metadata(str(self.source))
+        self.product.metadata.update(str(self.source))
 
         # For most Products, it's ok to do this check before
         # saving metadata, but not for GenericProduct, since the way
@@ -372,10 +379,11 @@ class Task(abc.ABC):
         self._logger.debug('Setting "%s" status to %s', self.name, value)
         self._exec_status = value
 
-        # some executors run this in a subprocess, the main process will not
-        # get the new metadata since it was not saved from there, reload
-        if value == TaskStatus.Executed:
-            self.product.metadata.load()
+        # some executors run this in a subprocess, the metadata in the main
+        # process becomes outdated, make sure you retrieve it again next time
+        # you need it
+        # if value == TaskStatus.Executed:
+        # self.product.metadata.clear()
 
         # process might crash, propagate now or changes might not be
         # reflected (e.g. if a Task is marked as Aborted, all downtream
@@ -428,8 +436,8 @@ class Task(abc.ABC):
         # this will interfer with other render calls
         self.render(force=force)
 
-        res = self._build(catch_exceptions=catch_exceptions)
-        self.product._clear_cached_status()
+        res, _ = self._build(catch_exceptions=catch_exceptions)
+        self.product.metadata.clear()
         return res
 
     def _build(self, catch_exceptions):
@@ -442,7 +450,7 @@ class Task(abc.ABC):
         if not catch_exceptions:
             res = self._run()
             self._run_on_finish()
-            return res
+            return res, self.product.metadata.to_dict()
         else:
             try:
                 res = self._run()
@@ -468,6 +476,8 @@ class Task(abc.ABC):
 
             if build_success:
                 try:
+                    # FIXME: move metadata saving and product checking,
+                    # the error message is misleading
                     # this not only runs the hook, but also
                     # calls save metadata and checks that the product exists
                     self._run_on_finish()
@@ -486,7 +496,7 @@ class Task(abc.ABC):
                 else:
                     self.exec_status = TaskStatus.Executed
 
-                return res
+                return res, self.product.metadata.to_dict()
             else:
                 try:
                     self._run_on_failure()
@@ -564,6 +574,12 @@ class Task(abc.ABC):
         outdated_by_code : str
             Factors to determine if Task.product is marked outdated when source
             code changes. Otherwise just the upstream timestamps are used.
+
+        Notes
+        -----
+        This method tries to avoid calls to check for product status whenever
+        possible (for efficiency), for example, when passing the force flag,
+        there is no need to check it
         """
         self._logger.debug('Calling render on task %s', self.name)
 
@@ -593,9 +609,10 @@ class Task(abc.ABC):
             # NOTE: is_outdated goes second so it's lazily evaluated
             if force or is_outdated.check():
                 self._exec_status = TaskStatus.WaitingExecution
-                self._logger.debug(
-                    'Forcing status "%s", outdated conditions'
-                    ' ignored...', self.name)
+                if force:
+                    self._logger.debug(
+                        'Forcing status "%s", outdated conditions'
+                        ' ignored...', self.name)
 
             else:
                 self._exec_status = TaskStatus.Skipped
@@ -718,6 +735,20 @@ class Task(abc.ABC):
                     product=str(self.product),
                     source_code=str(self.source))
 
+    def debug(self):
+        """Debug task, only implemented in certain tasks
+        """
+        raise NotImplementedError(
+            'debug is not implemented in "{}" tasks'.format(
+                type(self).__name__))
+
+    def develop(self):
+        """Develop task, only implemented in certain tasks
+        """
+        raise NotImplementedError(
+            'debug is not implemented in "{}" tasks'.format(
+                type(self).__name__))
+
     def _render_product(self):
         params_names = list(self.params)
 
