@@ -1,3 +1,4 @@
+import shlex
 import pdb
 import tempfile
 import subprocess
@@ -63,7 +64,7 @@ class NotebookConverter:
 
             exporter_name = suffix[1:]
 
-        self.exporter = self._get_exporter(exporter_name)
+        self.exporter = self._get_exporter(exporter_name, path_to_output)
         self.path_to_output = path_to_output
         self.nbconvert_export_kwargs = nbconvert_export_kwargs or {}
 
@@ -73,7 +74,7 @@ class NotebookConverter:
                              self.nbconvert_export_kwargs)
 
     @staticmethod
-    def _get_exporter(exporter_name):
+    def _get_exporter(exporter_name, path_to_output):
         """
         Get function to convert notebook to another format using nbconvert,
         first. In some cases, the exporter name matches the file extension
@@ -99,14 +100,19 @@ class NotebookConverter:
             # it raises ExporterNameError. However the exception is defined
             # since 5.6.1 so we can safely import it
             except (ValueError, ExporterNameError):
-                raise ValueError('Could not determine nbconvert exporter '
-                                 'with nane "{}" '
-                                 'either specify in the path extension '
-                                 'or pass a valid exporter name in '
-                                 'the NotebookRunner constructor, '
-                                 'valid exporters are: {}'.format(
+                raise ValueError('Could not find nbconvert exporter '
+                                 'with name "{}". '
+                                 'Either change the extension '
+                                 'or pass a valid "nbconvert_exporter_name" '
+                                 'value. Valid exporters are: {}. If "{}" '
+                                 'is not intended to be the output noteboook, '
+                                 'pass a dictionary with "nb" as the key '
+                                 'for the output notebook, and other '
+                                 'keys for the rest of the outputs'.format(
                                      exporter_name,
-                                     nbconvert.get_export_names()))
+                                     nbconvert.get_export_names(),
+                                     path_to_output,
+                                 ))
 
         return exporter
 
@@ -167,8 +173,8 @@ class NotebookRunner(Task):
         it is not needed unless the extension cannot be used to infer the
         final output format, in which case the nbconvert.get_exporter is used
     ext_in: str, optional
-        The input extension format. If source is a pathlib.Path, the extension
-        from there is used, if loaded from a str, this parameter is needed
+        Source extension. Required if loading from a str. If source is a
+        ``pathlib.Path``, the extension from the file is used.
     nb_product_key: str, optional
         If the notebook is expected to generate other products, pass the key
         to identify the output notebook (i.e. if product is a list with 3
@@ -186,8 +192,8 @@ class NotebookRunner(Task):
     nbconvert_export_kwargs : dict
         Keyword arguments to pass to the nbconvert.export function (this is
         only used if exporting the output ipynb notebook to another format).
-        You can use this for example to hide code cells using the exclude_input
-        parameter
+        You can use this, for example, to hide code cells using the
+        exclude_input parameter
     local_execution : bool, optional
         Change working directory to be the parent of the notebook's source.
         Defaults to False. This resembles the default behavior when
@@ -232,11 +238,12 @@ class NotebookRunner(Task):
 
         if isinstance(self.product, MetaProduct):
             if self.product.get(nb_product_key) is None:
-                raise KeyError('Key "{}" does not exist in product: {}. '
-                               'nb_product_key should be an existing '
-                               'key to know where to save the output '
-                               'notebook'.format(nb_product_key,
-                                                 str(self.product)))
+                raise KeyError("Key '{}' does not exist in product: {}. "
+                               "Either add it to specify the output notebook "
+                               "location or pass a 'nb_product_key' to the "
+                               "task constructor with the key that contains "
+                               " the output location".format(
+                                   nb_product_key, str(self.product)))
 
         if isinstance(self.product, MetaProduct):
             product_nb = (
@@ -261,7 +268,7 @@ class NotebookRunner(Task):
                               static_analysis=static_analysis,
                               **kwargs)
 
-    def develop(self, extra_jupyter_args=None):
+    def develop(self, app='notebook', args=None):
         """
         Opens the rendered notebook (with injected parameters) and adds a
         "debugging-settings" cell to the that changes directory to the current
@@ -274,8 +281,10 @@ class NotebookRunner(Task):
 
         Parameters
         ----------
-        extra_jupyter_args : list
-            Extra parameters passed to the "jupyter notebook" command
+        app : {'notebook', 'lab'}, default: 'notebook'
+            Which Jupyter application to use
+        args : str
+            Extra parameters passed to the jupyter application
 
         Notes
         -----
@@ -283,6 +292,22 @@ class NotebookRunner(Task):
         code will be updated only if the `hot_reload option` is turned on.
         See :class:`ploomber.DAGConfigurator` for details.
         """
+        # TODO: this code needs refactoring, should be a context manager
+        # like the one we have for PythonCallable.develop that abstracts
+        # the handling of the temporary notebook while editing
+
+        apps = {'notebook', 'lab'}
+
+        if app not in apps:
+            raise ValueError('"app" must be one of {}, got: "{}"'.format(
+                apps, app))
+
+        if self.source.language != 'python':
+            raise NotImplementedError(
+                'develop is not implemented for "{}" '
+                'notebooks, only python is supported'.format(
+                    self.source.language))
+
         if self.source.loc is None:
             raise ValueError('Can only use develop in notebooks loaded '
                              'from files, not from str')
@@ -297,7 +322,7 @@ class NotebookRunner(Task):
         tmp.write_text(content)
 
         # open notebook with injected debugging cell
-        _open_jupyter_notebook(str(tmp), extra_jupyter_args)
+        _open_jupyter_notebook(str(tmp), app=app, args=args)
 
         # read tmp file again, to see if the user made any changes
         content_new = Path(tmp).read_text()
@@ -330,6 +355,12 @@ class NotebookRunner(Task):
         Opens the notebook (with injected parameters) in debug mode in a
         temporary location
         """
+        if self.source.language != 'python':
+            raise NotImplementedError(
+                'debug is not implemented for "{}" '
+                'notebooks, only python is supported'.format(
+                    self.source.language))
+
         opts = {'ipdb', 'pdb', 'pm'}
 
         if kind not in opts:
@@ -384,6 +415,9 @@ class NotebookRunner(Task):
         if self.local_execution:
             self.papermill_params['cwd'] = str(self.source.loc.parent)
 
+        # create parent folders if they don't exist
+        Path(path_to_out_ipynb).parent.mkdir(parents=True, exist_ok=True)
+
         try:
             # no need to pass parameters, they are already there
             pm.execute_notebook(str(tmp), str(path_to_out_ipynb),
@@ -411,8 +445,8 @@ def _read_rendered_notebook(nb_str):
 
     source = """
 # Debugging settings (this cell will be removed before saving)
-# change the current working directory to the one when .debug() happen
-# to make relative paths work
+# change the current working directory to directory of the session that
+# invoked the jupyter app to make relative paths work
 from os import chdir
 chdir("{}")
 """.format(Path('.').resolve())
@@ -424,20 +458,20 @@ chdir("{}")
     return nb
 
 
-def _open_jupyter_notebook(path, extra_args=None):
+def _open_jupyter_notebook(path, app, args):
     """
     Open notebook using the jupyter notebook application
 
     Parameters
     ----------
-    extra_args : list
+    args : str
         List of extra arguments, executed command becomes:
-        "jupyter notebook {path} {extra_args}"
+        "jupyter notebook {path} {args}"
     """
-    extra_args = extra_args or []
+    args = [f'"{token}"' for token in shlex.split(args or '')]
 
     try:
-        subprocess.call(['jupyter', 'notebook', path] + extra_args)
+        subprocess.call(['jupyter', app, path] + args)
     except KeyboardInterrupt:
         print('Jupyter notebook server closed...')
 
