@@ -4,15 +4,19 @@ Task implementations
 A Task is a unit of work that produces a persistent change (Product)
 such as a bash or a SQL script
 """
+import shlex
 import subprocess
 import pdb
+
+from IPython.terminal.debugger import TerminalPdb, Pdb
+
 from ploomber.tasks.Task import Task
 from ploomber.sources import (PythonCallableSource, GenericSource, EmptySource)
 from ploomber.clients import ShellClient
 from ploomber.products.Metadata import MetadataAlwaysUpToDate
 from ploomber.exceptions import TaskBuildError
 from ploomber.constants import TaskStatus
-from ploomber.sources.debugging import CallableDebugger
+from ploomber.sources.interact import CallableInteractiveDeveloper
 from ploomber.tasks.Params import Params
 
 
@@ -86,19 +90,44 @@ class PythonCallable(Task):
             else:
                 self._serializer(out, params['product'])
 
-    def develop(self):
+    def develop(self, app='notebook', args=None):
+        """Edit function interactively using Jupyter
+
+        Parameters
+        ----------
+        app : str, {'notebook', 'lab'}, default='notebook'
+            Which jupyter application to use
+
+        args : str
+            Extra args passed to the selected jupyter application
+        """
+        apps = {'notebook', 'lab'}
+
+        if app not in apps:
+            raise ValueError('"app" must be one of {}, got: "{}"'.format(
+                apps, app))
+
+        if self.exec_status == TaskStatus.WaitingRender:
+            raise TaskBuildError(
+                'Error in task "{}". '
+                'Cannot call task.develop() on a task that has '
+                'not been '
+                'rendered, call DAG.render() first'.format(self.name))
+
         # TODO: Params should implement an option to call to_json_serializable
         # on product to avoid repetition I'm using this same code in notebook
         # runner. Also raise error if any of the params is not
         # json serializable
         # TODO: resolve to absolute to make relative paths work
-        params = self.params.to_dict()
+        params = self.params.to_json_serializable()
         params['product'] = params['product'].to_json_serializable()
 
-        # TODO: rename this, perhaps CallableInteractiveDeveloper?
-        with CallableDebugger(self.source.primitive, params) as tmp:
+        args = [f'"{token}"' for token in shlex.split(args or '')]
+
+        with CallableInteractiveDeveloper(self.source.primitive,
+                                          params) as tmp:
             try:
-                subprocess.call(['jupyter', 'notebook', tmp])
+                subprocess.call(['jupyter', app, tmp] + args)
             except KeyboardInterrupt:
                 print('Jupyter notebook server closed...')
 
@@ -115,17 +144,16 @@ class PythonCallable(Task):
         opts = {'ipdb', 'pdb'}
 
         if kind not in opts:
-            raise ValueError('kind must be one of {}'.format(opts))
+            raise ValueError('"kind" must be one of {}, got: "{}"'.format(
+                opts, kind))
 
         if self.exec_status == TaskStatus.WaitingRender:
             raise TaskBuildError('Error in task "{}". '
-                                 'Cannot debug task that has not been '
+                                 'Cannot call task.debug() on a task that has '
+                                 'not been '
                                  'rendered, call DAG.render() first'.format(
                                      self.name))
-
         if kind == 'ipdb':
-            from IPython.terminal.debugger import TerminalPdb, Pdb
-
             try:
                 # this seems to only work in a Terminal
                 ipdb = TerminalPdb()
@@ -242,12 +270,6 @@ class _Partition(Task):
     def _init_source(self, kwargs):
         return EmptySource(None, **kwargs)
 
-    def _null(self):
-        pass
-
-    def _false(self):
-        return False
-
 
 class _Gather(PythonCallable):
     def run(self):
@@ -308,10 +330,9 @@ class Link(Task):
     def _init_source(kwargs):
         return EmptySource(None, **kwargs)
 
-    def _null_save_metadata(self, metadata):
-        pass
-
     def _false(self):
+        # this should be __false but we can't due to
+        # https://bugs.python.org/issue33007
         return False
 
 
@@ -340,7 +361,7 @@ class Input(Task):
         super().__init__(product, dag, name, None)
 
         # do not save metadata (Product's location is read-only)
-        self.product._save_metadata = self._null_save_metadata
+        self.product.metadata.update = self._null_update_metadata
 
         # the product will always be considered outdated
         self.product._outdated_data_dependencies = self._true
@@ -362,8 +383,12 @@ class Input(Task):
     def _init_source(kwargs):
         return EmptySource(None, **kwargs)
 
-    def _null_save_metadata(self, metadata):
+    def _null_update_metadata(self, metadata):
+        # this should be __null_update_metadata but we can't due to
+        # https://bugs.python.org/issue33007
         pass
 
     def _true(self):
+        # this should be __true but we can't due to
+        # https://bugs.python.org/issue33007
         return True
